/*
Copyright (c) 2007,2007 Elizeu Nogueira da R. Jr.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, 
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution.
    * Neither the name of the Elizeu Nogueira da R. Jr. nor the names of its 
      contributors may be used to endorse or promote products derived from this 
      software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package br.com.dreamsource.mobile.jmesql.io;

/**
 *
 * @author Elizeu Nogueira da Rosa Jr.
 * @version 0.1
 *
 */

import com.sun.midp.rms.RecordStoreFile;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import br.com.dreamsource.mobile.jmesql.exceptions.SQLException;


public class LineNumberReader {
    
    private String name;
    private int position;
    private RecordStore Store;
    
    public LineNumberReader(String fname) throws SQLException {
        this.name = fname;
        position = 0;
        try {
            Store = RecordStore.openRecordStore(this.name, true);
        }
        catch (Exception e) {
            throw new SQLException(e.getMessage());
        }
    }
    
    
    public static boolean exists(String file) {
        try {
            RecordStore S = RecordStore.openRecordStore(file, false);
            S.closeRecordStore();
            return true;
        }
        catch (Exception e) {
            return false;
        }
    }
    
    public String readLine() throws SQLException {
        try {
            position++;
            if (Store.getNumRecords() < position) {
                return null;
            }
            return new String(Store.getRecord(position));
        }
        catch (Exception e) {
            throw new SQLException(e.getMessage());
        }
    }
    
    public void flush() {
        position = 0;
    }
    
    public void clear() throws SQLException {
        try {
            Store.closeRecordStore();
            RecordStore.deleteRecordStore(this.name);
            RecordStore.openRecordStore(this.name, true);
            position = 0;
        }
        catch (Exception e) {
            throw new SQLException (e.getMessage());
        }
    }
    
    public void deleteLine(String value) throws SQLException {
        try {
            for (int i = 0; i < Store.getNumRecords(); i++) {
                String str = new String(Store.getRecord(i + 1));
                if (str.indexOf(value) > -1) {
                    Store.deleteRecord(i + 1); 
                    break;
                }
            }
        } catch (Exception e) {
            throw new SQLException(e.getMessage());
        }
    } 
    
    public void updateLine(String value, String newS) throws SQLException {
        try {
            for (int i = 0; i < Store.getNumRecords(); i++) {
                String str = new String(Store.getRecord(i + 1));
                if (str.indexOf(value) > -1) {
                    String finalStr = "";
                    if (str.charAt(0) == '/') {
                        finalStr = str.substring(0, str.indexOf(value));
                    }
                    str = finalStr+newS;
                    Store.setRecord(i + 1, str.getBytes(), 0, str.getBytes().length); 
                    break;
                }
            }
        } catch (Exception e) {
            throw new SQLException(e.getMessage());
        }
    }
    
    public void writeLine(String value) throws SQLException {
        try {
            Store.addRecord(value.getBytes(), 0, value.getBytes().length);
        }
        catch (Exception e) {
            throw new SQLException(e.getMessage());
        }
    }
    
}

